@@ -21,7 +21,7 @@
 #include "Glob.h"
 #include "Getopt.h"
 #include "FTP.h"
-#include "Recent.h"
+#include "Bookmark.h"
 #include "Cpp.h"
 #include "Prefs.h"
 #include "Tips.h"
@@ -62,7 +62,7 @@ int gTrace = kTracingOff;
 extern int gNumCommands;
 extern Command gCommands[];
 extern int gVerbosity;
-extern RemoteSiteInfo gRmtInfo;
+extern Bookmark gRmtInfo;
 extern UserInfo gUserInfo;
 extern CppSymbol gCppSymbols[];
 extern int gNumCppSymbols;
@@ -105,6 +105,7 @@ int DoSystem(char *systType, size_t siz)
 /*ARGSUSED*/
 static void SigLocalPage(int sigNum)
 {
+	alarm(0);
 	longjmp(gShellJmp, 1);
 }	/* SigLocalPage */
 
@@ -234,9 +235,9 @@ void GetRemoteCWD(char *cdstub)
 			l = strchr(rp->msg.first->line, '"');
 			if ((l != NULL) && (l != r)) {
 				*r = '\0';
-				STRNCPY(gRemoteCWD, (++l));
+				++l;
+				STRNCPY(gRemoteCWD, l);
 				*r = '"';	/* Restore, so response prints correctly. */
-				SetScreenInfo();
 			}
 		} else {
 			/* xxxx is current directory.
@@ -246,9 +247,9 @@ void GetRemoteCWD(char *cdstub)
 				*r = '\0';
 				STRNCPY(gRemoteCWD, (rp->msg.first->line));
 				*r = ' ';	/* Restore, so response prints correctly. */
-				SetScreenInfo();
 			}
 		}
+		SetScreenInfo();
 	} else {
 		/* Error. */
 		if (cdstub != kDidNotChdir) {
@@ -721,11 +722,11 @@ void DoQuit(int exitStatus)
 	/* Only do this once, in case we get caught with infinite recursion. */
 	if (++gDoneApplication <= 1) {
 		if (gConnected)
-			DoClose();
+			DoClose(1);
 		(void) RunPrefixedMacro("quit.", "ncftp");
 		(void) RunPrefixedMacro("end.", "ncftp");
 		if (gOtherSessionRunning == 0) {
-			WriteRemoteInfoFile();
+			WriteBookmarkFile();
 			CloseLogs();
 			WritePrefs();
 			SaveHistory();
@@ -914,9 +915,9 @@ int VersionCmd(int argc, char **argv)
 	MultiLinePrintF("Version:       %s\n", gVersion);
 	MultiLinePrintF("Author:        Mike Gleason, NCEMRSoft (mgleason@cse.unl.edu)\n");
 #if (kBeta == 0) && (kAlpha == 0)
-	MultiLinePrintF("Archived at:   ftp://ftp.cs.unl.edu/pub/ncftp/ncftp.tgz\n");
+	MultiLinePrintF("Archived at:   ftp://ftp.probe.net/pub/ncftp/ncftp.tgz\n");
 #else
-	MultiLinePrintF("Archived in:   ftp://ftp.cs.unl.edu/pub/ncftp/BETA/\n");
+	MultiLinePrintF("Archived in:   ftp://ftp.probe.net/pub/ncftp/BETA/\n");
 #endif
 
 #ifdef __DATE__
@@ -1220,3 +1221,12 @@ int LookupCmd(int argc, char **argv)
 	}
 	return 0;
 }	/* LookupCmd */
+
+
+
+
+int BookmarkCmd(int argc, char **argv)
+{
+	SaveBookmark(argv[1]);
+	return 0;
+}	/* BookmarkCmd */
