@@ -28,13 +28,19 @@
 #	endif
 #endif
 
-#ifdef HAVE_LIBSOCKS
-#	define accept		Raccept
-#	define connect		Rconnect
-#	define getsockname	Rgetsockname
-#	define listen		Rlisten
+#ifdef HAVE_LIBSOCKS5
+#	define SOCKS 5
+#	include <socks.h>
+#else
+#	ifdef HAVE_LIBSOCKS
+#		define accept		Raccept
+#		define connect		Rconnect
+#		define getsockname	Rgetsockname
+#		define listen		Rlisten
+#	endif
 #endif
 
+
 /* The control stream we read responses to commands from. */
 FILE *gControlIn = NULL;
 
@@ -74,7 +80,7 @@ int gHasPASV;
 
 /* Need to special case if trying to read the startup message from the
  * server, so our command handler won't jump to the wrong spot if
- * the server hangsup at that point.
+ * the server hangs up at that point.
  */
 int gReadingStartup = 0;
 
@@ -92,7 +98,7 @@ string gActualHostName;
 /* Internet Protocol address of host we're connected to, as a string. */
 char gIPStr[32];
 
-extern jmp_buf gCmdLoopJmp;
+extern int gDoneApplication;
 
 /*
  * Some declare these with int's, others (hpux...) with size_t.
@@ -264,12 +270,12 @@ to be the full name of your hostname, i.e. -DHOSTNAME=\\\"cse.unl.edu\\\".\n");
 		DebugMsg("getdomainname did not return anything.\n");
 	else {
 /*
-# local domain names
-#
-# These can now be determined from the domainname system call.
-# The first component of the NIS domain name is stripped off unless
-# it begins with a dot or a plus sign.
-*/
+ * local domain names
+ *
+ * These can now be determined from the domainname system call.
+ * The first component of the NIS domain name is stripped off unless
+ * it begins with a dot or a plus sign.
+ */
 		if (domain[0] == '+')
 			domain[0] = '.';
 #		ifdef NIS_GETDOMAINNAME
@@ -580,6 +586,7 @@ int OpenControlConnection(char *host, unsigned int port)
 		DoneWithResponse(rp);
 		goto fatal;
 	}
+	gReadingStartup = 0;
 	DoneWithResponse(rp);
 
 	gHasPASV = 1;		/* Assume we have it, until proven otherwise. */
@@ -872,15 +879,20 @@ void SetPostHangupOnServerProc(HangupProc proc)
 
 void HangupOnServer(void)
 {
+	int wasConn;
+
+	wasConn = gConnected;
+
 	/* Since we want to close both sides of the connection for each
 	 * socket, we can just have them closed with close() instead of
 	 * using shutdown().
 	 */
-	CloseControlConnection();
-	CloseDataConnection();
-	if (gHangupProc != (HangupProc)0)
-		(*gHangupProc)();
-	longjmp(gCmdLoopJmp, 1);
+	if (wasConn != 0) {
+			CloseControlConnection();
+			CloseDataConnection();
+			if (gHangupProc != (HangupProc)0)
+				(*gHangupProc)();
+	}
 }	/* HangupOnServer */
 
 
