@@ -2,6 +2,8 @@
 
 #include "Sys.h"
 
+#include <signal.h>
+
 #ifdef HAVE_UTIME_H
 #	include <utime.h>
 #else
@@ -23,8 +25,8 @@ int gMayUTime = kDoUTime;	/* User variable. */
 extern longstring gPager;
 extern longstring gRemoteCWD;
 extern longstring gLocalCWD;
-extern int gTransferType;
-extern int gStdout;
+extern int gTransferType, gXferAbortFlag;
+extern int gStdout, gWinInit;
 extern char *gOptArg;
 extern int gOptInd;
 
@@ -258,7 +260,7 @@ int DoGet(GetOptionsPtr gopt)
 				 * should attempt to restart the transfer
 				 * from where we left off.
 				 */
-				if (st.st_size < fileSize) {
+				if ((st.st_size < fileSize) || (fileSize == kSizeUnknown)) {
 					restartPt = SZ(st.st_size);
 					mode = "a";
 					DebugMsg("Manually continuing local file %s.", local);
@@ -578,10 +580,14 @@ int GetDir(GetOptionsPtr gopt, char *dName, char *rRoot, char *lRoot)
 			rf = NULL;
 			lf = NULL;
 		}
+		if (gXferAbortFlag == SIGINT)
+			break;	/* Don't get rest of files if you interrupted. */
 	}
 	
 	/* Now get subdirectories. */
 	for (dirFile = dirFiles.first; dirFile != NULL; dirFile = dirFile->next) {
+		if (gXferAbortFlag == SIGINT)
+			break;	/* Don't get rest of files if you interrupted. */
 		fType = (int) dirFile->line[0];
 		if (fType == 'd') {
 			iName = dirFile->line + 1;
@@ -654,6 +660,8 @@ int DoGetWithGlobbingAndRecursion(GetOptionsPtr gopt)
 	for (globFile = globFiles.first; globFile != NULL;
 		globFile = globFile->next)
 	{
+		if (gXferAbortFlag == SIGINT)
+			break;	/* Don't get rest of files if you interrupted. */
 		if (gopt->recursive) {
 			fType = RemoteFileType(globFile->line);
 			if (fType == 'd') {
@@ -725,6 +733,8 @@ int GetCmd(int argc, char **argv)
 				gopt.lName = NULL;	/* Make it later. */
 			}
 			result = DoGet(&gopt);
+			if (gXferAbortFlag == SIGINT)
+				break;	/* Don't get rest of files if you interrupted. */
 			if (result < 0)
 				--errs;
 		}
@@ -732,6 +742,8 @@ int GetCmd(int argc, char **argv)
 		for (i=0; i<argc; i++) {
 			gopt.rName = argv[i];
 			errs += DoGetWithGlobbingAndRecursion(&gopt);
+			if (gXferAbortFlag == SIGINT)
+				break;	/* Don't get rest of files if you interrupted. */
 		}
 	}
 	
@@ -741,11 +753,81 @@ int GetCmd(int argc, char **argv)
 
 
 
+long CatGetProc(char *buf, size_t bufsize, XferSpecPtr xp)
+{
+	long len;
+
+	len = (long) BufferGets(buf, bufsize, xp);
+	return (len);
+}	/* CatGetProc */
+
+
+
+
+long CatPutProc(char *buf, size_t bufsize, XferSpecPtr xp)
+{
+	long result;
+	longstring buf2;
+
+	if (isatty(xp->outStream)) {
+		MakeStringPrintable(buf2, (unsigned char *) buf, sizeof(buf2));
+		MultiLinePrintF("%s", buf2);
+		result = (long) bufsize;
+	} else {
+		result = (long) write(xp->outStream, buf, bufsize);
+		if (result != bufsize) {
+			return (-1L);
+		}
+	}
+
+	return (result);
+}	/* CatPutProc */
+
+
+
+
+/* View a remote file through your pager. */
+int DoCat(char *remoteName)
+{
+	int result;
+	XferSpecPtr xp;
+
+	MultiLineInit();
+
+	xp = InitXferSpec();
+	xp->netMode = kNetReading;
+	xp->outStream = gStdout;
+	
+	/* This group is needed for the progress reporting and logging stuff.
+	 * Otherwise, it isn't that important.
+	 */
+	xp->doReports = kNoReports;
+	xp->localFileName = kLocalFileIsStdout;
+	xp->remoteFileName = remoteName;
+
+	/* This is supposed to be done previously, if you wanted accurate
+	 * file sizes from GetDateAndSize.
+	 */
+	SETASCII;
+	
+	/* Setup the parameter block to give to RDataCmd. */
+	xp->getBlock = CatGetProc;
+	xp->putBlock = CatPutProc;
+	/* xp->inStream = gDataSocket;  RDataCmd fills this in when it gets it. */
+
+	/* Send the request and do the transfer. */
+	result = RDataCmd(xp, "RETR %s", xp->remoteFileName);
+	DoneWithXferSpec(xp);
+	return (result);
+}	/* DoCat */
+
+
+
+
 /* We need to make something we can give to popen.  This is simple
  * if it is a plain file, but if they wanted to page a compressed
  * file we have to prepend the correct filter before the pager name.
  */
-static
 int MakePageCmdLine(char *cmd, size_t siz, char *remote_file)
 {
 	int useZCat;
@@ -823,12 +905,6 @@ int DoPage(char *remoteName)
 	int binaryPage;
 	XferSpecPtr xp;
 
-	if (gPager[0] == '\0') {
-		EPrintF("You haven't specified a program to use as a pager.\n");
-		EPrintF("You can set this from the preferences screen (prefs command).\n");
-		return -1;
-	}
-
 	binaryPage = MakePageCmdLine(pageCmd, sizeof(pageCmd), remoteName);
 	DebugMsg("%s page: %s\n",
 		binaryPage ? "Binary" : "Ascii",
@@ -876,19 +952,72 @@ int PageCmd(int argc, char **argv)
 	int i, result, errs;
 	LineList globFiles;
 	LinePtr globFile;
+	char *pagerProg;
 
-	for (i=1, errs=0; i<argc; i++) {
+	if (STREQ(argv[1], "-b") && (gWinInit > 0) && (argc > 2)) {
+		/* A hack to let you use the built-in pager like you
+		 * can with the lpage command.
+		 */
+		pagerProg = NULL;	/* Use built-in */
+		i = 2;
+	} else {
+		if (gPager[0] == '\0') {
+			EPrintF("You haven't specified a program to use as a pager.\n");
+			EPrintF("You can set this from the preferences screen (prefs command).\n");
+			return -1;
+		}
+		pagerProg = gPager;
+		i = 1;
+	}
+
+	for (errs=0; i<argc; i++) {
 		InitLineList(&globFiles);
 		RemoteGlob(&globFiles, argv[i], kListNoFlags);
 		for (globFile = globFiles.first; globFile != NULL;
 			globFile = globFile->next)
 		{
-			result = DoPage(globFile->line);
+			if (pagerProg == NULL)
+				result = DoCat(globFile->line);
+			else
+				result = DoPage(globFile->line);
 			if (result < 0)
 				--errs;
+			if (gXferAbortFlag == SIGINT)
+				break;	/* Don't get rest of files if you interrupted. */
 		}
 		DisposeLineListContents(&globFiles);
 	}
 
 	return (errs);
 }	/* PageCmd */
+
+
+
+
+/* View one or more remote files through your pager. */
+int CatCmd(int argc, char **argv)
+{
+	int i, result, errs;
+	LineList globFiles;
+	LinePtr globFile;
+
+	MultiLineInit();
+	for (i=1, errs=0; i<argc; i++) {
+		InitLineList(&globFiles);
+		RemoteGlob(&globFiles, argv[i], kListNoFlags);
+		for (globFile = globFiles.first; globFile != NULL;
+			globFile = globFile->next)
+		{
+			result = DoCat(globFile->line);
+			if (result < 0)
+				--errs;
+			if (gXferAbortFlag == SIGINT)
+				break;	/* Don't get rest of files if you interrupted. */
+			if (argc > 2)
+				MultiLinePrintF("### End of file %s ###\n", globFile->line);
+		}
+		DisposeLineListContents(&globFiles);
+	}
+
+	return (errs);
+}	/* CatCmd */
