@@ -9,7 +9,7 @@
 
 #include "Util.h"
 #include "Cmds.h"
-#include "Recent.h"
+#include "Bookmark.h"
 #include "Open.h"
 #include "Hostwin.h"
 
@@ -24,7 +24,7 @@
 WINDOW *gHostWin = NULL;
 
 /* This is a window devoted solely to serve as a scrolling list
- * of hosts.
+ * of bookmarks.
  */
 WINDOW *gHostListWin = NULL;
 
@@ -52,7 +52,7 @@ int gHilitedHost;
 /* This is a pointer to the actual information of the currently
  * selected host.
  */
-RemoteSiteInfoPtr gCurHostListItem;
+BookmarkPtr gCurHostListItem;
 
 /* How many lines compose a "page" in the host list's scrolling window. */
 int gHostListPageSize;
@@ -64,14 +64,14 @@ int gNeedToClearMsg = 0;
  * This is so we could restore the information if the user wanted to
  * abort the changes.
  */
-RemoteSiteInfo gEditRsi;
+Bookmark gEditRsi;
 
 jmp_buf gHostWinJmp;
 
-extern int gWinInit, gNumRecents, gRmtInfoIsNew;
-extern RemoteSiteInfoPtrList gHostNickNames;
-extern RemoteSiteInfoPtr gHosts;
-extern RemoteSiteInfo gRmtInfo;
+extern int gWinInit, gNumBookmarks, gRmtInfoIsNew;
+extern BookmarkPtrList gBookmarks;
+extern BookmarkPtr gHosts;
+extern Bookmark gRmtInfo;
 extern string gEmailAddress, gAnonPassword;
 extern unsigned int gFTPPort;
 
@@ -110,13 +110,13 @@ void UpdateHostWindows(int uptAll)
 
 
 
-/* This draws the scrolling list of hosts, and hilites the currently
+/* This draws the scrolling list of bookmarks, and hilites the currently
  * selected host.
  */
 void DrawHostList(void)
 {
 	int lastLine, i;
-	RemoteSiteInfoPtr rsip;
+	BookmarkPtr rsip;
 	string str;
 	int maxy, maxx;
 	int lmaxy, lmaxx;
@@ -126,10 +126,10 @@ void DrawHostList(void)
 	getmaxyx(gHostListWin, lmaxy, lmaxx);
 	getbegyx(gHostListWin, begy, begx);
 	getmaxyx(gHostWin, maxy, maxx);
-	/* We have a status line saying how many hosts there are in
+	/* We have a status line saying how many bookmarks there are in
 	 * the list.  That way the user knows something is supposed to
 	 * be there when the host list is totally empty, and also that
-	 * there are more hosts to look at when the entire host list
+	 * there are more bookmarks to look at when the entire host list
 	 * doesn't fit in the scroll window.
 	 */
 	WAttr(gHostWin, kUnderline, 1);
@@ -138,21 +138,21 @@ void DrawHostList(void)
 		begy - 1,
 		begx,
 		"%s",
-		"Number of hosts"
+		"Number of bookmarks"
 	);
 	WAttr(gHostWin, kUnderline, 0);
 	wprintw(
 		gHostWin,
 		": %3d",
-		gNumRecents
+		gNumBookmarks
 	);
 	sprintf(spec, "%%-16s %%-%ds", lmaxx - 17);
 	lastLine = lmaxy + gHostListWinStart;
-	for (i=gHostListWinStart; (i<lastLine) && (i<gNumRecents); i++) {
-		rsip = gHostNickNames[i];
+	for (i=gHostListWinStart; (i<lastLine) && (i<gNumBookmarks); i++) {
+		rsip = gBookmarks[i];
 		if (rsip == gCurHostListItem)
 			WAttr(gHostListWin, kReverse, 1);
-		sprintf(str, spec, rsip->nickName, rsip->name);
+		sprintf(str, spec, rsip->bookmarkName, rsip->name);
 		str[lmaxx] = '\0';
 		mvwaddstr(gHostListWin, i - gHostListWinStart, 0, str);
 		if (rsip == gCurHostListItem)
@@ -202,13 +202,13 @@ void NewHilitedHostIndex(int newIdx)
 {
 	int oldIdx, lastLine;
 
-	if (gNumRecents <= 0) {
+	if (gNumBookmarks <= 0) {
 		HostWinMsg(
-"No hosts in the list.  Try a /new, or open a site manually to add one.");
+"No bookmarks in the list.  Try a /new, or open a site manually to add one.");
 	} else {
 		oldIdx = gHilitedHost;
-		if (gNumRecents < gHostListPageSize)
-			lastLine = gHostListWinStart + gNumRecents - 1;
+		if (gNumBookmarks < gHostListPageSize)
+			lastLine = gHostListWinStart + gNumBookmarks - 1;
 		else
 			lastLine = gHostListWinStart + gHostListPageSize - 1;
 		if (newIdx < gHostListWinStart) {
@@ -221,8 +221,8 @@ void NewHilitedHostIndex(int newIdx)
 			gHilitedHost = gHostListWinStart = newIdx;
 		} else if (newIdx > lastLine) {
 			/* Will need to scroll the window down. */
-			if (newIdx > (gNumRecents - 1)) {
-				newIdx = gNumRecents - 1;
+			if (newIdx > (gNumBookmarks - 1)) {
+				newIdx = gNumBookmarks - 1;
 				if (oldIdx == newIdx)
 					HostWinMsg("You are at the bottom of the list.");
 			}
@@ -234,7 +234,7 @@ void NewHilitedHostIndex(int newIdx)
 			/* Don't need to scroll window, just move pointer. */
 			gHilitedHost = newIdx;
 		}
-		gCurHostListItem = gHostNickNames[gHilitedHost];
+		gCurHostListItem = gBookmarks[gHilitedHost];
 		if (oldIdx != newIdx)
 			DrawHostList();
 	}
@@ -251,11 +251,11 @@ void HostWinZoomTo(int c)
 {	
 	int i, j;
 
-	if (gNumRecents > 0) {
+	if (gNumBookmarks > 0) {
 		if (islower(c))
 			c = toupper(c);
-		for (i=0; i<gNumRecents - 1; i++) {
-			j = gHostNickNames[i]->nickName[0];
+		for (i=0; i<gNumBookmarks - 1; i++) {
+			j = gBookmarks[i]->bookmarkName[0];
 			if (islower(j))
 				j = toupper(j);
 			if (j >= c)
@@ -263,7 +263,7 @@ void HostWinZoomTo(int c)
 		}
 		NewHilitedHostIndex(i);
 	} else {
-		HostWinMsg("No sites to select.  Try a /new.");
+		HostWinMsg("No bookmarks to select.  Try a /new.");
 	}
 	DrawHostList();
 }	/* HostWinZoomTo */
@@ -304,13 +304,13 @@ void HostListPageDown(void)
 
 
 
-/* This marks the start of a section that belongs to the Site Options
+/* This marks the start of a section that belongs to the Bookmark Options
  * window.  This window pops up on top of the host editor's main window
  * when you wish to edit a site's settings.  When the user finishes,
  * we close it and the host editor resumes.
  */
 
-/* This displays a message in the Site Options window. */
+/* This displays a message in the Bookmark Options window. */
 void EditHostWinMsg(char *msg)
 {
 	int maxy, maxx;
@@ -419,7 +419,6 @@ void EditHostWinDraw(int flags, int hilite)
 	string str;
 	char spec[32];
 	char *cp;
-	size_t len;
 
 	/* Draw the keys the user can type in reverse text. */
 	WAttr(gEditHostWin, kReverse, 1);
@@ -449,8 +448,8 @@ void EditHostWinDraw(int flags, int hilite)
 	/* Now draw the items on a case-by-case basis. */
 	if (TESTBIT(flags, kNicknameEditWindowItem)) {
 		mvwprintw(gEditHostWin, kNicknameEditWindowItem + f, 3, spec,
-			"Nickname:",
-			gEditRsi.nickName
+			"Bookmark name:",
+			gEditRsi.bookmarkName
 		);
 		wclrtoeol(gEditHostWin);
 	}
@@ -649,7 +648,7 @@ void HostWinEdit(void)
 
 		/* leaveok(gEditHostWin, TRUE);	* Not sure if I like this... */
 		WAttr(gEditHostWin, kBold, 1);
-		WAddCenteredStr(gEditHostWin, 0, "Site Options");
+		WAddCenteredStr(gEditHostWin, 0, "Bookmark Options");
 		WAttr(gEditHostWin, kBold, 0);
 		
 		/* We'll be editing a copy of the current host's settings. */
@@ -711,8 +710,8 @@ void HostWinEdit(void)
 			EditHostWinDraw(BIT(field), kHilite);
 			switch(field) {
 				case kNicknameEditWindowItem:
-					EditHostWinMsg("Type a new nickname, or hit <RETURN> to continue.");
-					EditHostWinGetStr(gEditRsi.nickName, sizeof(gEditRsi.nickName), kNotOkayIfEmpty, kGetAndEcho);
+					EditHostWinMsg("Type a new bookmark name, or hit <RETURN> to continue.");
+					EditHostWinGetStr(gEditRsi.bookmarkName, sizeof(gEditRsi.bookmarkName), kNotOkayIfEmpty, kGetAndEcho);
 					break;
 					
 				case kHostnameEditWindowItem:
@@ -785,7 +784,7 @@ void HostWinEdit(void)
 		delwin(gEditHostWin);
 		gEditHostWin = NULL;
 		*gCurHostListItem = gEditRsi;
-		SortNickNames();
+		SortBookmarks();
 		NewHilitedHostIndex(gCurHostListItem->index);
 		UpdateHostWindows(1);
 	}
@@ -797,7 +796,7 @@ void HostWinEdit(void)
 void HostWinDup(void)
 {
 	if (gCurHostListItem != NULL) {
-		gCurHostListItem = DuplicateRemoteInfo(gCurHostListItem);
+		gCurHostListItem = DuplicateBookmark(gCurHostListItem);
 		gHilitedHost = gCurHostListItem->index;
 	} else
 		HostWinMsg("Nothing to duplicate.");
@@ -810,15 +809,15 @@ void HostWinDup(void)
 /* Removes a site from the host list. */
 void HostWinDelete(void)
 {
-	RemoteSiteInfoPtr toDelete;
+	BookmarkPtr toDelete;
 	int newi;
 	
 	if (gCurHostListItem != NULL) {
 		toDelete = gCurHostListItem;
 
 		/* Need to choose a new active host after deletion. */
-		if (gHilitedHost == gNumRecents - 1) {
-			if (gNumRecents == 1) {
+		if (gHilitedHost == gNumBookmarks - 1) {
+			if (gNumBookmarks == 1) {
 				newi = -1;	/* None left. */
 			} else {
 				/* At last one before delete. */
@@ -830,11 +829,11 @@ void HostWinDelete(void)
 			 */
 			newi = gHilitedHost;
 		}
-		DeleteRemoteInfo(toDelete);
+		DeleteBookmark(toDelete);
 		if (newi < 0)
 			gCurHostListItem = NULL;
 		else {
-			gCurHostListItem = gHostNickNames[newi];
+			gCurHostListItem = gBookmarks[newi];
 			gHilitedHost = newi;
 		}
 	} else
@@ -848,13 +847,13 @@ void HostWinDelete(void)
 /* Adds a new site to the host list, with default settings in place. */
 void HostWinNew(void)
 {
-	RemoteSiteInfoPtr rsip;
-	RemoteSiteInfo rsi;
+	BookmarkPtr rsip;
+	Bookmark rsi;
 
-	SetNewRemoteInfoDefaults(&rsi);
-	rsip = AddRemoteSiteInfoPtr(&rsi);
+	SetNewBookmarkDefaults(&rsi);
+	rsip = AddBookmarkPtr(&rsi);
 	rsip->port = gFTPPort;
-	SortNickNames();
+	SortBookmarks();
 	gCurHostListItem = rsip;
 	gHilitedHost = rsip->index;
 	gHostListWinStart = rsip->index - gHostListPageSize + 1;
@@ -921,6 +920,7 @@ void HostWinGetStr(char *str, size_t size)
 /*ARGSUSED*/
 void SigIntHostWin(int sigNum)
 {
+	alarm(0);
 	longjmp(gHostWinJmp, 1);
 }	/* SigIntHostWin */
 
@@ -936,7 +936,7 @@ int HostWindow(void)
 #ifdef USE_CURSES
 	int c;
 	string cmd;
-	volatile RemoteSiteInfoPtr toOpen;
+	volatile BookmarkPtr toOpen;
 	VSig_t si;
 	int maxy, maxx;
 	int lmaxy, lmaxx;
@@ -970,10 +970,10 @@ int HostWindow(void)
 			 */
 			gHostListWinStart = 0;
 			gHilitedHost = 0;
-			if (gNumRecents == 0)
+			if (gNumBookmarks == 0)
 				gCurHostListItem = NULL;
 			else
-				gCurHostListItem = gHostNickNames[gHilitedHost];
+				gCurHostListItem = gBookmarks[gHilitedHost];
 			
 			/* Initially, we don't want to connect to any site in
 			 * the host list.
@@ -981,21 +981,21 @@ int HostWindow(void)
 			toOpen = NULL;
 	
 			WAttr(gHostWin, kBold, 1);
-			WAddCenteredStr(gHostWin, 0, "Host Editor");
+			WAddCenteredStr(gHostWin, 0, "Bookmark Editor");
 			WAttr(gHostWin, kBold, 0);
 			
-			mvwaddstr(gHostWin, 3, 2, "Open selected site:      <enter>");
-			mvwaddstr(gHostWin, 4, 2, "Edit selected site:      /ed");
-			mvwaddstr(gHostWin, 5, 2, "Delete selected site:    /del");
-			mvwaddstr(gHostWin, 6, 2, "Duplicate selected site: /dup");
-			mvwaddstr(gHostWin, 7, 2, "Add a new site:          /new");
-			mvwaddstr(gHostWin, 9, 2, "Up one:                  <u>");
-			mvwaddstr(gHostWin, 10, 2, "Down one:                <d>");
-			mvwaddstr(gHostWin, 11, 2, "Previous page:           <p>");
-			mvwaddstr(gHostWin, 12, 2, "Next page:               <n>");
+			mvwaddstr(gHostWin, 3, 2, "Open selected site:       <enter>");
+			mvwaddstr(gHostWin, 4, 2, "Edit selected site:       /ed");
+			mvwaddstr(gHostWin, 5, 2, "Delete selected site:     /del");
+			mvwaddstr(gHostWin, 6, 2, "Duplicate selected site:  /dup");
+			mvwaddstr(gHostWin, 7, 2, "Add a new site:           /new");
+			mvwaddstr(gHostWin, 9, 2, "Up one:                   <u>");
+			mvwaddstr(gHostWin, 10, 2, "Down one:                 <d>");
+			mvwaddstr(gHostWin, 11, 2, "Previous page:            <p>");
+			mvwaddstr(gHostWin, 12, 2, "Next page:                <n>");
 			mvwaddstr(gHostWin, 14, 2, "Capital letters selects first");
 			mvwaddstr(gHostWin, 15, 2, "  site starting with the letter.");
-			mvwaddstr(gHostWin, 17, 2, "Exit the host editor:    <x>");
+			mvwaddstr(gHostWin, 17, 2, "Exit the bookmark editor: <x>");
 			
 			/* Initialize the scrolling host list window. */
 			gHostListWin = subwin(
@@ -1039,7 +1039,7 @@ int HostWindow(void)
 					else if (ISTREQ(cmd, "new"))
 						HostWinNew();
 					else
-						HostWinMsg("Invalid host editor command.");
+						HostWinMsg("Invalid bookmark editor command.");
 				} else switch(c) {
 					case 10:	/* ^J == newline */
 					case 13:	/* ^M == carriage return */
@@ -1049,7 +1049,7 @@ int HostWindow(void)
 						if (gCurHostListItem == NULL)
 							HostWinMsg("Nothing to open.  Try 'open sitename' from the main screen.");
 						else {
-							toOpen = (volatile RemoteSiteInfoPtr) gCurHostListItem;
+							toOpen = (volatile BookmarkPtr) gCurHostListItem;
 							goto done;
 						}
 						break;
@@ -1120,11 +1120,11 @@ done:
 		flushinp();
 		if (si != (Sig_t) kNoSignalHandler)
 			SIGNAL(SIGINT, si);
-		if (toOpen != (volatile RemoteSiteInfoPtr) 0) {
+		if (toOpen != (volatile BookmarkPtr) 0) {
 			/* If the user selected a site to open, connect to it now. */
 			InitOpenOptions(&openopt);
-			STRNCPY(openopt.hostname, ((RemoteSiteInfoPtr) toOpen)->nickName);
-			GetRemoteInfo(openopt.hostname, sizeof(openopt.hostname));
+			STRNCPY(openopt.hostname, ((BookmarkPtr) toOpen)->bookmarkName);
+			GetBookmark(openopt.hostname, sizeof(openopt.hostname));
 			openopt.port = gRmtInfo.port;
 			Beep(0);    /* Reset beep timer. */
 			return (Open(&openopt));
