@@ -21,8 +21,10 @@
 
 #include <pwd.h>
 #include <errno.h>
+#include <ctype.h>
 #include <signal.h>
 #include <setjmp.h>
+#include <stdlib.h>
 
 #include "Util.h"
 #include "Main.h"
@@ -30,11 +32,13 @@
 #include "Open.h"
 #include "Cmdline.h"
 #include "DateSize.h"
-#include "Recent.h"
+#include "Bookmark.h"
 #include "Prefs.h"
 #include "FTP.h"
+#include "Glob.h"
 #include "Getopt.h"
 #include "Xfer.h"
+#include "Complete.h"
 #include "Tips.h"
 #include "Version.h"
 
@@ -326,6 +330,9 @@ void OpenTraceLog(void)
 	if (gTraceLogFile != NULL)
 		return;	/* Already open. */
 
+	if (gOurDirectoryPath[0] == '\0')
+		return;		/* Don't create in root directory. */
+
 	OurDirectoryPath(traceLogPath, sizeof(traceLogPath), kTraceLogName);	
 	if (access(traceLogPath, F_OK) == 0) {
 		/* Need to prepend last session's log to the master debug log. */
@@ -435,6 +442,7 @@ void InitOurDirectory(void)
 		/* Don't create it if you're root and your home directory
 		 * is the root directory.
 		 */
+		gOurDirectoryPath[0] = '\0';
 		return;
 	} else {
 		(void) Path(gOurDirectoryPath,
@@ -622,7 +630,7 @@ void Init(void)
 	ReadPrefs();
 
 	OpenLogs();
-	ReadRemoteInfoFile();
+	ReadBookmarkFile();
 	ReadMacroFile();
 	InitCommandAndMacroNameList();
 	
@@ -672,7 +680,10 @@ void CloseLogs(void)
 		return;
 
 	CloseFile(&gLogFile);
-	
+
+	if (gOurDirectoryPath[0] == '\0')
+		return;		/* Don't create in root directory. */
+
 	/* If the user wants to, s/he can specify the maximum size of the log file,
 	 * so it doesn't waste too much disk space.  If the log is too fat, trim the
 	 * older lines (at the top) until we're under the limit.
@@ -738,6 +749,8 @@ void SaveHistory(void)
 	int i;
 
 	if (gWinInit) {
+		if (gOurDirectoryPath[0] == '\0')
+			return;		/* Don't create in root directory. */
 		lp = gCmdHistory.last;
 		OurDirectoryPath(histFileName, sizeof(histFileName), kHistoryName);
 		fp = fopen(histFileName, "w");
@@ -797,12 +810,13 @@ void StartupMsgs(void)
 
 	if (gOtherSessionRunning) {
 		BoldPrintF("Note: Another session is running. Changes made to the\n");
-		BoldPrintF("      prefs, hosts, and trace files will not be saved.\n\n");
+		BoldPrintF("      prefs, bookmarks, and trace files will not be saved.\n\n");
 	}
 
 	(void) GetCWD(curLocalCWD, sizeof(curLocalCWD));
 	if (gDownloadDir[0] != '\0') {
 		/* If set, always try to cd there. */
+		ExpandTilde(gDownloadDir, sizeof(gDownloadDir));
 		(void) chdir(gDownloadDir);	/* May not succeed. */
 	} else if (gRememberLCWD) {
 		/* Try restoring the last local directory we were in.
@@ -828,6 +842,7 @@ void Startup(void)
 		InitWindows();
 		StartupMsgs();
 		(void) RunPrefixedMacro("start.", "ncftp");
+ 		InitReadline();
 	}
 }	/* Startup */
 
@@ -839,6 +854,7 @@ static void
 SigIntMain(int sigNum)
 {
 	SIGNAL(SIGINT, SigIntMain);
+	alarm(0);
 	EPrintF("\n*Interrupt*\n");
 	longjmp(gMainJmp, 1);
 }   /* SigIntMain */
@@ -868,7 +884,7 @@ void main(int argc, char **argv)
 	 * sets of flags.
 	 */
 	while ((opt = Getopt(argc, argv, "aiup:rd:g:cmCfGRn:zDLVH")) >= 0) {
-		if (strchr("aiup:rd:g:cmfGRn:z", opt) == NULL) {
+		if (strchr("aiup:rd:g:cmCfGRn:z", opt) == NULL) {
 			switch (opt) {
 				case 'D':
 					gDebug = kDebuggingOn;
