@@ -11,13 +11,14 @@
 #include "GetPass.h"
 #include "Cmds.h"
 #include "RCmd.h"
-#include "Recent.h"
+#include "Bookmark.h"
 #include "FTP.h"
 #include "Get.h"
 #include "Getopt.h"
 #include "Macro.h"
 #include "Hostwin.h"
 #include "Main.h"
+#include "Complete.h"
 
 /* This is a preference setting specifying whether we use anonymous logins
  * by default (as opposed to user/passwd logins).
@@ -28,7 +29,7 @@ int gAnonOpen = (UOPEN == 0);
  * the remote host, so we don't need to find out the hard way each
  * time we do something.
  */
-extern RemoteSiteInfo gRmtInfo;
+extern Bookmark gRmtInfo;
 
 /* Name of the host we're connected to. */
 string gHost;
@@ -38,6 +39,19 @@ string gHost;
  */
 int gLoggedIn = 0;
 
+/* Need to know if we are trying to connect and login, so if we get
+ * hangup during that, we will come back to the open without longjmp'ing
+ * out.
+ */
+int gAttemptingConnection = 0;
+
+/* Need to know if they did a 'open -r' so we don't use the built-in pager
+ * for long site-connect messages.  Otherwise the program would stop to
+ * wait for the user to hit <space> each time we connected to that site
+ * which would sort of defeat the purpose of redialing.
+ */
+int gRedialModeEnabled = 0;
+
 /* If we connect successfully, we'll want to save this port number
  * when we add a host file entry.
  */
@@ -56,7 +70,7 @@ extern int gTransferType, gCurType;
 extern int gRmtInfoIsNew, gWantRmtInfoSaved;
 extern int gConnected, gHasPASV, gVerbosity;
 extern int gDoneApplication, gWinInit;
-extern string gEmailAddress, gAnonPassword;
+extern string gEmailAddress, gAnonPassword, gPager;
 extern UserInfo gUserInfo;
 extern FILE *gLogFile;
 extern LineList gRedir;
@@ -120,7 +134,7 @@ int Login(char *u, char *p, char *a)
 				goto okay;
 
 			case 421:	/* 421 Service not available, closing control connection. */
-				break;
+				goto done;
 				
 			case 331:	/* 331 User name okay, need password. */
 				isAnonLogin =	STREQ("anonymous", u2) ||
@@ -153,7 +167,13 @@ int Login(char *u, char *p, char *a)
 				goto done;
 				
 			default:
-				Error(kDontPerror, "Unexpected response: %s\n", rp->msg.first->line);
+				if (rp->msg.first == NULL) {
+					Error(kDontPerror, "Lost connection during login.\n");
+				} else {
+					Error(kDontPerror, "Unexpected response: %s\n",
+						rp->msg.first->line
+					);
+				}
 				goto done;
 		}
 	}
@@ -195,31 +215,30 @@ void PostCloseStuff(void)
 		STRNCPY(gRmtInfo.lastIP, gIPStr);
 		if (gEventNumber > 0L) {
 			/* Only do these if we were not in batch mode (colon-mode). */
-			STRNCPY(gRmtInfo.dir, gRemoteCWD);
-			(void) RunPrefixedMacro("close.", gRmtInfo.nickName);
+			if (gRmtInfo.noSaveDir == 0)
+					STRNCPY(gRmtInfo.dir, gRemoteCWD);
+			(void) RunPrefixedMacro("close.", gRmtInfo.bookmarkName);
 			(void) RunPrefixedMacro("close.", "any");
 		} else {
 			/* Only do these if we are running colon mode. */
-			(void) RunPrefixedMacro("colon.close.", gRmtInfo.nickName);
+			(void) RunPrefixedMacro("colon.close.", gRmtInfo.bookmarkName);
 			(void) RunPrefixedMacro("colon.close.", "any");
 		}
-		SaveRemoteInfo(NULL);
+		SaveCurHostBookmark(NULL);
 	}
 	gLoggedIn = 0;
 	gRemoteCWD[0] = gHost[0] = '\0';
-	if (gDoneApplication == 0)	
-		SetScreenInfo();
 }	/* PostCloseStuff */
 
 
 
 
 /* Close the connection to the remote host and cleanup. */
-void DoClose(void)
+void DoClose(int tryQUIT)
 {
 	ResponsePtr rp;
 	
-	if (gConnected) {
+	if (tryQUIT != 0) {
 		rp = InitResponse();
 		rp->eofOkay = 1;	/* We are expecting EOF after this cmd. */
 		RCmd(rp, "QUIT");
@@ -235,7 +254,9 @@ void DoClose(void)
 
 int CloseCmd(int argc, char **argv)
 {
-	DoClose();
+	DoClose(gConnected);
+	if (gWinInit == 0)
+			SetScreenInfo();	/* Need for line mode. */
 	return kNoErr;
 }	/* CloseCmd */
 
@@ -256,17 +277,17 @@ void InitOpenOptions(OpenOptions *openopt)
 	openopt->openmode = gAnonOpen ? kOpenImplicitAnon : kOpenImplicitUser;
 
 	/* Normally you don't want to ignore the entry in your netrc. */
-	openopt->ignore_rc = 0;
+	openopt->ignoreRC = 0;
 
 	/* Set the default delay if the user specifies redial mode without
 	 * specifying the redial delay.
 	 */
-	openopt->redial_delay = kRedialDelay;
+	openopt->redialDelay = kRedialDelay;
 
 	/* Normally, you only want to try once. If you specify redial mode,
 	 * this is changed.
 	 */
-	openopt->max_dials = 1;
+	openopt->maxDials = 1;
 	
 	/* You don't want to dump the file to stdout by default. */
 	openopt->ftpcat = kNoFTPCat;
@@ -277,7 +298,7 @@ void InitOpenOptions(OpenOptions *openopt)
 	openopt->port = kPortUnset;
 
 	/* We are not in colon-mode (yet). */
-	openopt->colonmodepath[0] = 0;
+	openopt->colonModePath[0] = 0;
 
 	/* Set the hostname to a null string, since there is no default host. */
 	openopt->hostname[0] = 0;
@@ -325,7 +346,7 @@ int GetOpenOptions(int argc, char **argv, OpenOptions *openopt, int fromMain)
 
 			case 'i':
 				/* User wants to ignore the entry in the netrc. */
-				openopt->ignore_rc = 1;
+				openopt->ignoreRC = 1;
 				break;
 
 			case 'p':
@@ -345,18 +366,18 @@ int GetOpenOptions(int argc, char **argv, OpenOptions *openopt, int fromMain)
 				/* User supplied a delay (in seconds) that differs from
 				 * the default.
 				 */
-				openopt->redial_delay = atoi(gOptArg);
+				openopt->redialDelay = atoi(gOptArg);
 				break;
 				
 			case 'g':
 				/* User supplied an upper-bound on the number of redials
 				 * to try.
 				 */
-				openopt->max_dials = atoi(gOptArg);
+				openopt->maxDials = atoi(gOptArg);
 				break;
 
 			case 'r':
-				openopt->max_dials = -1;
+				openopt->maxDials = -1;
 				break;
 
 			case 'm':
@@ -458,10 +479,10 @@ Try 'ncftp -c wuarchive.wustl.edu:/README > file.'\n");
 					www = 1;
 				}
 				if (cpath != NULL) {
-					(void) STRNCPY(openopt->colonmodepath, www ? "/" : "");
-					(void) STRNCAT(openopt->colonmodepath, cpath);
-					cp = openopt->colonmodepath +
-						strlen(openopt->colonmodepath) - 1;
+					(void) STRNCPY(openopt->colonModePath, www ? "/" : "");
+					(void) STRNCAT(openopt->colonModePath, cpath);
+					cp = openopt->colonModePath +
+						strlen(openopt->colonModePath) - 1;
 					if (*cp == '/') {
 						/* Colon-mode path ended in a slash, so you said it
 						 * was a directory.  That means we should start from
@@ -470,7 +491,7 @@ Try 'ncftp -c wuarchive.wustl.edu:/README > file.'\n");
 						*cp = '\0';
 						openopt->interactiveColonMode = 1;
 					}
-					DebugMsg("Colon-Mode Path = '%s'\n", openopt->colonmodepath);
+					DebugMsg("Colon-Mode Path = '%s'\n", openopt->colonModePath);
 				}
 
 			} else {
@@ -481,7 +502,7 @@ Try 'ncftp -c wuarchive.wustl.edu:/README > file.'\n");
 		}
 		(void) STRNCPY(openopt->hostname, hostp);
 
-		if ((openopt->colonmodepath[0] == '\0') &&
+		if ((openopt->colonModePath[0] == '\0') &&
 			(openopt->ftpcat != kNoFTPCat))
 		{
 			/* User specified ftpcat mode, but didn't supply
@@ -493,7 +514,7 @@ ncftp -c wuarchive.wustl.edu:/pub/README (to cat this file to stdout).\n");
 			return kUsageErr;
 		}
 
-		GetRemoteInfo(openopt->hostname, sizeof(openopt->hostname));
+		GetBookmark(openopt->hostname, sizeof(openopt->hostname));
 		if (openopt->port == kPortUnset) {
 			/* No port specified, so use same port as last time. */
 			if (gRmtInfo.port == kPortUnset)
@@ -564,18 +585,18 @@ void ColonMode(OpenOptions *openopt)
 	 */
 	if (openopt->ftpcat == kFTPCat) {
 		InitGetOutputMode(&openopt->gopt, kDumpToStdout);
-		openopt->gopt.rName = openopt->colonmodepath;
+		openopt->gopt.rName = openopt->colonModePath;
 		openopt->gopt.doReports = 0;
 		result = DoGet(&openopt->gopt);
 	} else if (openopt->ftpcat == kFTPMore) {
-		result = DoPage(openopt->colonmodepath);
+		result = DoPage(openopt->colonModePath);
 	} else {
 		/* Regular colon-mode, where we fetch the file, putting the
 		 * copy in the current local directory.
 		 */
 		InitGetOutputMode(&openopt->gopt, kSaveToDisk);
 		openopt->gopt.doReports = 0;
-		openopt->gopt.rName = openopt->colonmodepath;
+		openopt->gopt.rName = openopt->colonModePath;
 		result = DoGetWithGlobbingAndRecursion(&openopt->gopt);
 	}
 	DoQuit(result == 0 ? result : kExitColonModeFail);
@@ -627,7 +648,7 @@ void PostLoginStuff(OpenOptions *openopt)
 		fprintf(gLogFile, "%s at %s", gHost, ctime(&now));
 	}
 
-	wasColonMode = openopt->colonmodepath[0] != (char)0;
+	wasColonMode = openopt->colonModePath[0] != (char)0;
 
 	/* We need to check for unix and see if we should set binary
 	 * mode automatically.
@@ -640,7 +661,7 @@ void PostLoginStuff(OpenOptions *openopt)
 			 * gave us was a directory, and we should just cd to that
 			 * directory when we start up.
 			 */
-			(void) DoChdir(openopt->colonmodepath);
+			(void) DoChdir(openopt->colonModePath);
 		} else {
 			/* Regular colon-mode is fetching a file by specifying the
 			 * pathname of the file on the shell command line.
@@ -659,19 +680,19 @@ void PostLoginStuff(OpenOptions *openopt)
 		/* Freshen 'cwd' variable for the prompt. */
 		(void) GetRemoteCWD(kDidNotChdir);
 	}
-	
-	/* This updates the prompt, and for visual mode, the status bar. */
-	SetScreenInfo();
 
 	if (wasColonMode) {
 		/* Run separate sets of macros for colon-mode opens and regular,
 		 * interactive opens.
 		 */
 		(void) RunPrefixedMacro("colon.open.", "any");
-		(void) RunPrefixedMacro("colon.open.", gRmtInfo.nickName);
+		(void) RunPrefixedMacro("colon.open.", gRmtInfo.bookmarkName);
 	} else {
 		(void) RunPrefixedMacro("open.", "any");
-		(void) RunPrefixedMacro("open.", gRmtInfo.nickName);
+		(void) RunPrefixedMacro("open.", gRmtInfo.bookmarkName);
+		if (gWinInit == 0)
+				SetScreenInfo();	/* Need for line mode. */
+		ClearDirCache();
 	}
 }	/* PostLoginStuff */
 
@@ -696,6 +717,8 @@ int Open(OpenOptions *openopt)
 	 */
 	ResetBlockTimeout();
 
+	gRedialModeEnabled = (openopt->maxDials != 1);
+
 	if (ISEXPLICITOPEN(openopt->openmode)) {
 		/* User specified an open mode explictly, like open -u,
 		 * so do what the user said in spite of what we may have had
@@ -730,7 +753,7 @@ int Open(OpenOptions *openopt)
 		r_acct = gRmtInfo.acct;
 	}
 
-	if ((openopt->colonmodepath[0]) && (!openopt->interactiveColonMode)) {
+	if ((openopt->colonModePath[0]) && (!openopt->interactiveColonMode)) {
 		/* If we're running from a shell script (or whatever)
 		 * don't dump any output.  If the user is doing this from
 		 * the shell, we will at least print the error messages.
@@ -739,7 +762,7 @@ int Open(OpenOptions *openopt)
 		 * going to print anything but error messages anyway,
 		 * we're okay by using kErrorsOnly.
 		 */
-		if (gIsToTTY)
+		if (gIsToTTY != 0)
 			(void) SetVerbose(kErrorsOnly);
 		else
 			(void) SetVerbose(kQuiet);
@@ -756,24 +779,26 @@ int Open(OpenOptions *openopt)
 	PrintF("Trying to connect to %s...\n", openopt->hostname);
 	for (
 			dials = 0;
-			openopt->max_dials < 0 || dials < openopt->max_dials;
+			openopt->maxDials < 0 || dials < openopt->maxDials;
 			dials++)
 	{
 		if (dials > 0) {
 			/* If this is the second dial or higher, sleep a bit. */
 			PrintF("Sleeping %u seconds...  ",
-				(unsigned) openopt->redial_delay);
+				(unsigned) openopt->redialDelay);
 			FlushListWindow();
-			(void) sleep((unsigned) openopt->redial_delay);
+			(void) sleep((unsigned) openopt->redialDelay);
 			PrintF("Retry Number: %d\n", dials + 1);
 		}
 		FlushListWindow();
 
+		SetBar(NULL, "CONNECTING", NULL, 1, 1);
+		gAttemptingConnection = 1;
 		hErr = OpenControlConnection(openopt->hostname, openopt->port);
 		if (hErr == kConnectErrFatal) {
 			/* Irrecoverable error, so don't bother redialing.
 			 * The error message should have already been printed
-			 * from Hookup().
+			 * from OpenControlConnection().
 			 */
 			DebugMsg("Cannot recover from open error %d.\n", hErr);
 			break;
@@ -783,18 +808,20 @@ int Open(OpenOptions *openopt)
 			gRemoteCWD[0] = '\0';
 	
 			/* This updates the status bar (for visual mode). */
-			SetScreenInfo();
+			SetBar(NULL, "LOGGING IN", NULL, 1, 1);
 			SetPostHangupOnServerProc(PostCloseStuff);
 			loginResult = Login(user, pass, r_acct);
 			
 			if (loginResult == 0) {
 				PostLoginStuff(openopt);
-				if (openopt->max_dials != 1) {
+				if (openopt->maxDials != 1) {
 					/* If they selected redial mode, beep at the user
 					 * to get their attention.
 					 */
 					Beep(1);
 				}
+				gAttemptingConnection = 0;	/* We are connected. */
+				gRedialModeEnabled = 0;		/* Not dialing any more. */
 				return(kNoErr);	/* Login okay, so done. */
 			}
 			/* Otherwise, an error during login occurred, so try again. */
@@ -803,10 +830,14 @@ int Open(OpenOptions *openopt)
 			FlushListWindow();
 		}
 		
-		DoClose();
+		DoClose(gConnected);
+		gAttemptingConnection = 0;
 	}
 
-	if ((openopt->colonmodepath[0]) && (!openopt->interactiveColonMode)) {
+	/* Display error now. */
+	FlushListWindow();
+
+	if ((openopt->colonModePath[0]) && (!openopt->interactiveColonMode)) {
 		/* If we get here, we we're colon-moding and got a non-redialable
 		 * error or we ran out of attempts.
 		 */
@@ -826,7 +857,7 @@ int OpenCmd(int argc, char **argv)
 	/* If there is already a site open, close that one so we can
 	 * open a new one.
 	 */
-	DoClose();
+	DoClose(gConnected);
 
 	if (argc < 2) {
 		result = HostWindow();
