@@ -10,15 +10,15 @@
 /* Variables for Open() that can be changed from the command line. */
 typedef struct OpenOptions {
 	int				openmode;
-	int				ignore_rc;
+	int				ignoreRC;
 	unsigned int	port;
-	int				redial_delay;
-	int				max_dials;
+	int				redialDelay;
+	int				maxDials;
 	int				ftpcat;
-	int				login_verbosity;
+	int				loginVerbosity;
 	char			hostname[128];
 	char			cdpath[256];
-	char			colonmodepath[256];
+	char			colonModePath[256];
 	int				interactiveColonMode;
 	GetOptions		gopt;
 } OpenOptions;
@@ -45,7 +45,7 @@ typedef struct OpenOptions {
 int LoginQuestion(char *, char *, size_t, char *, int);
 int Login(char *, char *, char *);
 void PostCloseStuff(void);
-void DoClose(void);
+void DoClose(int);
 int CloseCmd(int, char **);
 void InitOpenOptions(OpenOptions *);
 int CheckForColonMode(OpenOptions *);
