@@ -13,7 +13,7 @@
 #include "Hostwin.h"
 #include "Prefs.h"
 #include "RCmd.h"
-#include "Recent.h"
+#include "Bookmark.h"
 #include "Main.h"
 
 #ifdef USE_CURSES
@@ -36,7 +36,7 @@ jmp_buf gPrefsWinJmp;
 extern int gWinInit;
 extern string gAnonPassword;
 extern int gMayUTime, gStartupMsgs, gRemoteMsgs, gBlankLines;
-extern int gAnonOpen, gWhichProgMeter, gMaxLogSize, gMaxRecents;
+extern int gAnonOpen, gWhichProgMeter, gMaxLogSize, gMaxBookmarks;
 extern int gDefaultVisualMode, gTotalRuns, gRememberLCWD;
 extern int gNetworkTimeout, gTrace, gPreferredDataPortMode;
 extern longstring gLocalCWD, gPager, gDownloadDir;
@@ -44,6 +44,7 @@ extern long gTotalXferKiloBytes, gTotalXferHSeconds;
 extern int gMarkTrailingSpace;
 
 
+
 PrefOpt gPrefOpts[] = {
 	{ "anonopen", "Default open mode:",
 		kToggleMsg,
@@ -60,9 +61,9 @@ PrefOpt gPrefOpts[] = {
 	{ "logsize", "User log size:",
 		"Enter the maximum number of bytes allowed for log file, or 0 for no log.",
 		PREFINT(gMaxLogSize) },
-	{ "maxhosts", "Max hosts to save:",
-		"Enter the maximum number of hosts allowed for host file, or 0 for no limit.",
-		PREFINT(gMaxRecents) },
+	{ "maxbookmarks", "Max bookmarks to save:",
+		"Enter the max number of bookmarks for bookmarks file, or 0 for no limit.",
+		PREFINT(gMaxBookmarks) },
 	{ "pager", "Pager:",
 		"Type the pathname of the program you use to view text a page at a time.",
 		PREFSTR(gPager, kOkayIfEmpty, kGetAndEcho) },
@@ -112,7 +113,7 @@ PrefOpt gNonEditPrefOpts[] = {
 		PREFINT(gTotalXferKiloBytes) },
 };
 
-#define kNumEditablePrefOpts ((int)(sizeof(gPrefOpts) / sizeof(PrefOpt)))
+int gNumEditablePrefOpts = ((int)(sizeof(gPrefOpts) / sizeof(PrefOpt)));
 #define kNumNonEditablePrefOpts ((int)(sizeof(gNonEditPrefOpts) / sizeof(PrefOpt)))
 
 
@@ -173,11 +174,11 @@ void GetPrefSetting(char *dst, size_t siz, int item)
 			(void) Strncpy(dst, str, siz);
 			break;
 
-		case kMaxHostsWinItem:
-			if (gMaxRecents == kNoRecentLimit)
+		case kMaxBookmarksWinItem:
+			if (gMaxBookmarks == kNoBookmarkLimit)
 				(void) Strncpy(str, "unlimited", siz);
 			else
-				sprintf(str, "%d", gMaxRecents);
+				sprintf(str, "%d", gMaxBookmarks);
 			(void) Strncpy(dst, str, siz);
 			break;
 
@@ -473,6 +474,7 @@ void PrefsWinToggle(int *val, int bitNum, int min, int max)
 /*ARGSUSED*/
 void SigIntPrefsWin(int sigNum)
 {
+	alarm(0);
 	longjmp(gPrefsWinJmp, 1);
 }	/* SigIntPrefsWin */
 
@@ -698,6 +700,9 @@ void WritePrefs(void)
 	FILE *fp;
 	int i;
 
+	if (gOurDirectoryPath[0] == '\0')
+		return;		/* Don't create in root directory. */
+
 	OurDirectoryPath(path, sizeof(path), kPrefsName);
 	fp = fopen(path, "w");
 	if (fp == NULL) {
@@ -770,7 +775,7 @@ void ReadPrefs(void)
 			continue;
 		o = strlen(option) + 1;
 		STRNCPY(val, line + o);
-		pop = (PrefOpt *) BSEARCH(option, gPrefOpts, SZ(kNumEditablePrefOpts),
+		pop = (PrefOpt *) BSEARCH(option, gPrefOpts, SZ(gNumEditablePrefOpts),
 			sizeof(PrefOpt), PrefSearchProc);
 		if (pop == NULL) {
 			pop = (PrefOpt *) BSEARCH(option, gNonEditPrefOpts,
