@@ -8,7 +8,7 @@
 #include "Xfer.h"
 #include "Progress.h"
 #include "GetPass.h"
-#include "Recent.h"
+#include "Bookmark.h"
 #include "Main.h"
 
 /* NOTE: There is more commentary at the bottom of this source file. */
@@ -44,7 +44,7 @@ static int gStoppedInProgressReport;
 extern int gVerbosity, gIsFromTTY, gScreenWidth;
 extern FILE *gLogFile;
 extern longstring gRemoteCWD;
-extern RemoteSiteInfo gRmtInfo;
+extern Bookmark gRmtInfo;
 extern int gConsecutiveTimeouts;
 
 #ifdef USE_CURSES
@@ -218,8 +218,8 @@ int PrPhilBar(XferSpecPtr xp, int mode)
 			secsLeft = (int) ((xp->bytesLeft / xp->bytesPerSec) + 0.5);
 			minLeft = secsLeft / 60;
 			secsLeft = secsLeft - (minLeft * 60);
-			if (minLeft > 99) {
-				minLeft = 99;
+			if (minLeft > 999) {
+				minLeft = 999;
 				secsLeft = 59;
 			}
 			
@@ -321,8 +321,8 @@ int PrStatBar(XferSpecPtr xp, int mode)
 			secLeft = (int) ((xp->bytesLeft / xp->bytesPerSec) + 0.5);
 			minLeft = secLeft / 60;
 			secLeft = secLeft - (minLeft * 60);
-			if (minLeft > 99) {
-				minLeft = 99;
+			if (minLeft > 999) {
+				minLeft = 999;
 				secLeft = 59;
 			}
 
@@ -332,7 +332,7 @@ int PrStatBar(XferSpecPtr xp, int mode)
 			AbbrevStr(localName, xp->localFileName, sizeof(localName) - 2, 0);
 			STRNCAT(localName, ":");
 
-			sprintf(line, "%-32s   %5.1f/%.1f %s   %5.1f %s/s   ETA %2d:%02d%s",
+			sprintf(line, "%-32s   %5.1f/%.1f %s   %5.1f %s/s   ETA %3d:%02d%s",
 				localName,
 				done,
 				uTotal,
@@ -449,9 +449,10 @@ int StartProgress(XferSpecPtr xp)
 	 * do reports if the xp said explicitly not to do.  For example,
 	 * we don't want any reports for directory listings.
 	 */
-	if ((gVerbosity == kQuiet) || (!xp->doReports) || (!InForeGround()))
+	if ((gVerbosity == kQuiet) || (!xp->doReports) || (!InForeGround())) {
 		progMeterInUse = kPrNone;
-	else {
+		xp->doReports = 0;
+	} else {
 		progMeterInUse =  gWhichProgMeter;
 		
 		/* If the progress meter number is out of range, use a default value. */
@@ -546,7 +547,8 @@ void ProgressReport(XferSpecPtr xp, int forceUpdate)
 				 * the ^Z, doing the progress report, then sending us the
 				 * suspend signal again.
 				 */
-				if ((sts = SIGNAL(SIGTSTP, SIG_IGN)) != SIG_IGN) {
+				sts = SIGNAL(SIGTSTP, SIG_IGN);
+				if (sts != SIG_IGN) {
 					gStoppedInProgressReport = 0;
 					SIGNAL(SIGTSTP, ProgressSuspsend);
 				}
@@ -589,9 +591,11 @@ void ProgressReport(XferSpecPtr xp, int forceUpdate)
 				/* Compute the next update time. */
 				xp->timeOfNextUpdate = xp->endTime.tv_sec + kDelaySeconds - 1;
 #ifdef SIGTSTP
-				SIGNAL(SIGTSTP, sts);
-				if (gStoppedInProgressReport)
-					kill(getpid(), SIGTSTP);
+				if (sts != SIG_IGN) {
+					SIGNAL(SIGTSTP, sts);
+					if (gStoppedInProgressReport != 0)
+						kill(getpid(), SIGTSTP);
+				}
 #endif
 				/* Figure out how long the transfer has been going. */
 			} else {
@@ -629,9 +633,6 @@ void EndProgress(XferSpecPtr xp)
 			((*xp->prProc)(xp, kPrEndMsg) == kPrWantStatsMsg);
 	}
 	(void) Gettimeofday(&xp->endTime);
-	
-	/* Make sure echoing is back on! */
-	Echo(stdin, 1);
 
 	/* Compute transfer stats. */
 	xRate = TransferRate(
@@ -663,6 +664,9 @@ void EndProgress(XferSpecPtr xp)
 			);
 		}
 		STRNCAT(statMsg, ".");
+	
+		/* Make sure echoing is back on! */
+		Echo(stdin, 1);
 
 		/* Make sure the rest of the line is padded with spaces, so it will
 		 * erase junk that may have been leftover from a progress meter.
