@@ -30,10 +30,11 @@ int gNetworkTimeout = kDefaultNetworkTimeout;
 jmp_buf	gTStrTimeOut;
 
 extern FILE *gControlIn, *gControlOut;
-extern int gDataSocket;
+extern int gDataSocket, gDoneApplication;
 extern int gDebug, gConnected, gTrace;
 extern jmp_buf gCmdLoopJmp;
 extern int gPreferredDataPortMode, gReadingStartup;
+extern int gAttemptingConnection;
 
 /* The user can control the level of output printed by the program
  * with this.
@@ -234,6 +235,7 @@ void ReInitResponse(ResponsePtr rp)
 static
 void TimeoutGetTelnetString(int unused)
 {
+	alarm(0);
 	longjmp(gTStrTimeOut, 1);
 }	/* TimeoutGetTelnetString */
 
@@ -250,10 +252,25 @@ int GetTelnetString(char *str, size_t siz, FILE *cin, FILE *cout)
 	int eofError;
 	char *cp;
 
+	if ((cin == NULL) || (cout == NULL)) {
+		eofError = -1;
+		goto done;
+	}
+
 	if (setjmp(gTStrTimeOut) != 0) {
 		Error(kDontPerror, "Host is not responding to commands, hanging up.\n");
-		HangupOnServer();
-		/*NOTREACHED*/
+		if (gAttemptingConnection == 0) {
+			HangupOnServer();
+			if (!gDoneApplication) {
+				alarm(0);
+				longjmp(gCmdLoopJmp, 1);
+			}
+		} else {
+			/* Give up and return to Open(). */
+			DoClose(0);
+		}
+		eofError = -1;
+		goto done;
 	}
 	SIGNAL(SIGALRM, TimeoutGetTelnetString);
 	alarm(gNetworkTimeout);
@@ -425,16 +442,19 @@ eof:
 			if (*str)
 				AddLine(&rp->msg, str);
 eofMsg:
-			if (!rp->eofOkay)
+			if (rp->eofOkay == 0)
 				Error(kDontPerror, "Remote host has closed the connection.\n");
 			rp->hadEof = 1;
-			gConnected = 0;
-			DoClose();
-			if (!gReadingStartup) {
+			if (gAttemptingConnection == 0) {
 				HangupOnServer();
-				/*NOTREACHED*/
-			}
-			DoClose();
+				if (!gDoneApplication) {
+					alarm(0);
+					longjmp(gCmdLoopJmp, 1);
+				}
+			} else if (rp->eofOkay == 0) {
+				/* Give up and return to Open(). */
+				DoClose(0);
+			}	/* else rp->eofOkay, which meant we already closed. */
 			return (5);
 		}
 		cp = str;
