@@ -5,11 +5,12 @@
 
 #include <errno.h>
 #include <signal.h>
+#include <stdlib.h>
 
 #include "Util.h"
 #include "Main.h"
 #include "Version.h"
-#include "Recent.h"
+#include "Bookmark.h"
 #include "RCmd.h"
 #include "LGets.h"
 #include "GetPass.h"
@@ -20,6 +21,7 @@
 #include "WGets.h"
 
 extern int endwin(void);
+extern long gEventNumber;
 WINDOW *gListWin;
 WINDOW *gInputWin;
 WINDOW *gPromptWin;
@@ -29,6 +31,9 @@ int gPageNum;
 int gMultiLineMode = 0;
 int gPendingNL = 0;
 int gBackgroundProcessing = 0;
+int gUsingDefaultBar = 0;
+int gUsingTmpCenterBar = 0;
+string gBarLeft, gBarCenter, gBarRight;
 
 /* Other protos, whose parameters aren't WINDOW *ptrs, are in Win.h */
 void WAddCenteredStr(WINDOW *, int, char *);
@@ -46,13 +51,14 @@ int gScreenWidth = 80;
 extern longstring gRemoteCWD;
 extern longstring gLockFileName;
 extern int gConnected, gVerbosity;
-extern RemoteSiteInfo gRmtInfo;
+extern Bookmark gRmtInfo;
 extern string gHost;
 extern int gStdout, gRealStdout, gOtherSessionRunning;
 extern FILE *gTraceLogFile;
 extern int gDebug, gTrace;
 extern LineList gCmdHistory;
 extern int gVisualMode, gIsToTTY, gIsFromTTY, gStartup;
+extern int gRedialModeEnabled;
 
 
 void EndWin(void)
@@ -60,8 +66,10 @@ void EndWin(void)
 	if (gWinInit) {
 		gWinInit = 0;
 #ifdef USE_CURSES
-		clear();
-		refresh();
+		if (gEventNumber > 0L) {
+			clear();
+			refresh();
+		}
 		endwin();
 #endif	/* USE_CURSES */
 	}
@@ -81,6 +89,15 @@ void Exit(int exitStatus)
 
 
 
+static void
+SigTerm(int sigNum)
+{
+	Exit(kExitSignal);
+}	/* SigTerm */
+
+
+
+
 void SaveScreen(void)
 {
 #ifdef USE_CURSES
@@ -268,8 +285,6 @@ void WAttr(WINDOW *w, int attr, int on)
 void UpdateScreen(int wholeScreen)
 {
 #ifdef USE_CURSES
-	int i;
-
 	if (gWinInit) {
 		if (wholeScreen) {
 			touchwin(gListWin);
@@ -316,15 +331,18 @@ void SigTStp(int sigNum)
 			/* Doing this with these windows open can cause problems. */
 			return;
 		}
-		SIGNAL(SIGTSTP, SIG_DFL);
 		SaveScreen();
-		TraceMsg("\nSuspended.\n");
+		TraceMsg("SIGTSTP: Suspended.\n");
+		SIGNAL(SIGTSTP, SIG_DFL);
 		kill(getpid(), SIGSTOP);	/* Send stop signal to ourselves. */
 	} else {
 		SIGNAL(SIGTSTP, SigTStp);
 		SIGNAL(SIGCONT, SigTStp);
-		TraceMsg("\nResumed.\n");
-		sleep(1);
+		if (sigNum == SIGCONT) {
+			TraceMsg("SIGCONT: Resumed.\n");
+		} else {
+			TraceMsg("SIG %d.\n", sigNum);
+		}
 		if (InForeGround())
 			RestoreScreen(0);
 		else
@@ -350,7 +368,20 @@ char *Gets(char *str, size_t size)
 
 #ifdef USE_CURSES
 	if (gWinInit) {
+		/* Background processing only gets turned on if you use ^Z
+		 * from within the program and then "bg" it.  We assume that
+		 * when you do that, you want to get a "tty output" message
+		 * when the operation finishes.  Note that you wouldn't get
+		 * gBackgroundProcessing == 1 if you did "ncftp &" because
+		 * you would never get to the part that turns it on (the ^Z
+		 * handler).
+		 */
 		if (gBackgroundProcessing) {
+			/* Want to give you a "stopped - tty output" message in
+			 * your shell when we get here.
+			 */
+			PrintF("\nBackground processing has finished.\n");
+			FlushListWindow();
 			while (! InForeGround())
 				sleep(1);
 			gBackgroundProcessing = 0;
@@ -366,6 +397,7 @@ char *Gets(char *str, size_t size)
 			werase(gInputWin);
 			wnoutrefresh(gInputWin);
 		}
+		SetScreenInfo();
 		doupdate();
 		wgp.w = gInputWin;
 		wgp.sy = 0;
@@ -432,77 +464,125 @@ void GetAnswer(char *prompt, char *answer, size_t siz, int noEcho)
 
 
 
-void SetBarLeftRight(char *l, char *r, int prefSide)
+void SetBar(char *l, char *c, char *r, int doUp, int tmpCenter)
 {
 #ifdef USE_CURSES
-	int llen, rlen;
 	int maxy, maxx;
 	int i;
+	int rmax;
+	size_t llen, rlen, clen;
+	string bar;
+	string barTmp;
 
 	if (gWinInit) {
-		llen = (int) strlen(l);
-		rlen = (int) strlen(r);
-		
+		if (l == NULL)
+			l = gBarLeft;
+		else if (doUp != -1)
+			STRNCPY(gBarLeft, l);	
+
+		if (r == NULL)
+			r = gBarRight;
+		else if (doUp != -1)
+			STRNCPY(gBarRight, r);	
+
+		if (c == NULL) {
+			c = gBarCenter;
+	/*		if (gUsingDefaultBar == 1) {
+				*c = '\0';
+				gUsingDefaultBar = 0;
+			} else */ if (gUsingTmpCenterBar == 1) {
+				*c = '\0';
+				gUsingTmpCenterBar = 0;
+			}
+		} else {
+			STRNCPY(gBarCenter, c);	
+			gUsingTmpCenterBar = 0;
+			if (tmpCenter)
+				gUsingTmpCenterBar = 1;
+		}
+
 		getmaxyx(gBarWin, maxy, maxx);
-		if (llen + rlen > maxx - 1)
-			SetBarCentered(prefSide == 'R' ? r : l);
+		for (i=0; i<maxx; i++)
+			bar[i] = ' ';
+
+		llen = strlen(l);
+		if (llen > maxx - 1)
+			llen = maxx - 1;
+		rlen = strlen(r);
+		if (rlen > maxx - 1)
+			rlen = maxx - 1;
+		clen = strlen(c);
+		if (clen > maxx - 1)
+			clen = maxx - 1;
+
+		if /* (rlen + (clen/2) > (maxx/2)) */
+			((2*rlen) + (clen) > (maxx))
+		{
+				/* Put the center part on the left so we can see it. */
+				memcpy(bar, c, clen);
+			
+				/* Do the right side. */	
+				rmax = maxx - 1 - clen;
+				if (rmax > 0) {
+					AbbrevStr(barTmp, r, (size_t) rmax, 0);
+					rmax = strlen(barTmp);
+					memcpy(bar + maxx - rmax, r, rmax);
+				}
+		} else {
+				/* Do the left side. */	
+				memcpy(bar, l, llen);
+
+				/* Do the middle. */	
+				if (*c != '\0') {
+					rlen = maxx - 1;
+					AbbrevStr(barTmp, c, (size_t) rlen, 0);
+					rlen = strlen(barTmp);
+					memcpy(bar + (maxx - 1 - rlen) / 2, barTmp, rlen);
+				}
+			
+				/* Do the right side. */	
+				rmax = maxx - 1 - llen;
+				if (rmax > 0) {
+					AbbrevStr(barTmp, r, (size_t) rmax, 0);
+					rmax = strlen(barTmp);
+					memcpy(bar + maxx - rmax, r, rmax);
+				}
+		}
+
+		bar[maxx] = '\0';
 		
 		wmove(gBarWin, 0, 0);
-		waddstr(gBarWin, l);
-		
-		i = maxx - llen - rlen;
-		while (--i >= 0)
-			waddch(gBarWin, ' ');
-		waddstr(gBarWin, r);
-		wnoutrefresh(gBarWin);
+		waddstr(gBarWin, bar);
+
+		if (doUp == 0)	
+			wnoutrefresh(gBarWin);
+		else
+			wrefresh(gBarWin);
 	}
 #endif	/* USE_CURSES */
-}	/* SetBarLeftRight */
-
-
-
-
-void SetBarCentered(char *s)
-{
-#ifdef USE_CURSES
-	int len;
-	int i, j;
-	int maxy, maxx;
-
-	if (gWinInit) {
-		len = (int) strlen(s);
-		
-		wmove(gBarWin, 0, 0);
-		getmaxyx(gBarWin, maxy, maxx);
-		i = (maxx - len) / 2;
-		if (i < 0)
-			i = 0;
-		j = i;
-		while (--j >= 0)
-			waddch(gBarWin, ' ');
-		waddstr(gBarWin, s);
-		for (i += len; i < maxx; ++i)
-			waddch(gBarWin, ' ');
-		wnoutrefresh(gBarWin);
-	}
-#endif	/* USE_CURSES */
-}	/* SetBarCentered */
+}	/* SetBar */
 
 
 
 
 void SetDefaultBar(void)
 {
+#ifdef USE_CURSES
 	string str;
 
 	if (gWinInit) {
 		STRNCPY(str, "NcFTP ");
 		STRNCAT(str, kVersion);
-		STRNCAT(str, " by Mike Gleason, NCEMRSoft (mgleason@cse.unl.edu).");
-		SetBarCentered(str);
+		STRNCAT(str, " by Mike Gleason, NCEMRSoft (mgleason@probe.net).");
+		gUsingDefaultBar = 1;
+
+		SetBar("", str, "", 0, 0);
 	} else {
 		SetPrompt(kUseDefaultPrompt);
 	}
+#else
+	SetPrompt(kUseDefaultPrompt);
+#endif
 }	/* SetDefaultBar */
 
 
@@ -521,15 +601,15 @@ void SetScreenInfo(void)
 		if (gConnected) {
 			maxPCwdLen = gScreenWidth - strlen(gRmtInfo.name) - 2;
 			AbbrevStr(pr, pcwd, maxPCwdLen, 0);
-			SetBarLeftRight(gRmtInfo.name, pr, 'R');
-			SetPrompt(gRmtInfo.nickName);
+			SetBar(gRmtInfo.name, NULL, pr, 0, 0);
+			SetPrompt(gRmtInfo.bookmarkName);
 		} else {
 			SetDefaultBar();
 			SetPrompt(kUseDefaultPrompt);
 		}
 	} else {
 		if (gConnected) {
-			STRNCPY(pr, gRmtInfo.nickName);
+			STRNCPY(pr, gRmtInfo.bookmarkName);
 			STRNCAT(pr, ":");
 			len = (int) strlen(pr);
 			len2 = (int) strlen(pcwd);
@@ -559,8 +639,16 @@ void PrintToListWindow(char *buf, int multi)
 	int y, x;
 	string pr;
 
-	if ((multi) && (gSkipToEnd == 1))
-		return;
+	if (multi != 0) {
+		if (gSkipToEnd == 1)
+			return;
+		/* Don't wait between pages if they are redialing.  We don't
+		 * want to page the site's connect message if redialing is
+		 * turned on.
+		 */
+		if (gRedialModeEnabled != 0)
+			multi = 0;
+	}
 
 	endp = buf;
 	startp = buf;
@@ -1081,7 +1169,6 @@ void InitWindows(void)
 	char *cp;
 	int on;
 #ifdef USE_CURSES
-	int i;
 	int maxx, maxy;
 #endif
 
@@ -1093,6 +1180,7 @@ void InitWindows(void)
 		if (stdscr == NULL)
 			goto fail;
 		gWinInit = 1;
+		SIGNAL(SIGTERM, SigTerm);
 
 		nl();
 
@@ -1113,6 +1201,10 @@ void InitWindows(void)
 		getmaxyx(gListWin, maxy, maxx);
 		gScreenWidth = maxx + 1;
 
+		gBarLeft[0] = '\0';
+		gBarCenter[0] = '\0';
+		gBarRight[0] = '\0';
+
 		LoadHistory();
 
 		/* Probably not set yet, unless you specified a host on
