@@ -2,9 +2,11 @@
 
 #include "Sys.h"
 
+#ifndef NeXT
 #ifndef _POSIX_SOURCE
 #	define _POSIX_SOURCE 1
 #endif
+#endif
 
 #ifndef _POSIX_C_SOURCE
 #	define _POSIX_C_SOURCE 3	/* For solaris only? */
@@ -31,13 +33,8 @@ char *gSecondaryBuf = NULL;
 /* Size of the transfer buffer.  */
 size_t gXferBufSize = kXferBufSize;
 
-/* A copy of the current transfer information block, in case we get
- * interrupted during the transfer itself.
- */
-XferSpecPtr gCurXp;
-
 /* Stores whether we had an interrupt occur during the transfer. */
-static int gAbortFlag;
+int gXferAbortFlag = 0;
 
 char *gSecondaryBufPtr;
 char *gSecondaryBufLimit;
@@ -82,10 +79,13 @@ void InitXferBuffer(void)
 
 
 
+#ifndef ReadOrTimeout
 static void IOAlarm(int sigNum)
 {
+	alarm(0);
 	longjmp(gXferTimeoutJmp, 1);
 }	/* IOAlarm */
+#endif
 
 
 
@@ -218,7 +218,11 @@ done:
 /* We get here upon a signal we can handle during transfers. */
 void XferSigHandler(int sigNum)
 {
-	gAbortFlag = sigNum;
+	gXferAbortFlag = sigNum;
+#if 1
+	/* Not a good thing to do in general from a signal handler... */
+	TraceMsg("XferSigHandler: SIG %d.\n", sigNum);
+#endif
 	return;
 }	/* XferSigHandler */
 
@@ -284,7 +288,7 @@ void AbortDataTransfer(XferSpecPtr xp)
 	}
 #else
 	/* Fake it by just reading until EOF. */
-	while (read(xp->inStream, buf, sizeof(buf))
+	while (read(xp->inStream, buf, sizeof(buf) > 0)
 		;
 #endif
 	CloseDataConnection();
@@ -323,7 +327,7 @@ int DataTransfer(XferSpecPtr xp)
 	put = xp->putBlock;
 	in = xp->inStream;
 	out = xp->outStream;
-	gAbortFlag = 0;
+	gXferAbortFlag = 0;
 
 	/* In case we happen to use the secondary I/O buffer (i.e. want to use
 	 * BufferGets), this line sets the buffer pointer so that the first thing
@@ -358,7 +362,7 @@ int DataTransfer(XferSpecPtr xp)
 			gConsecutiveTimeouts < kMaxConsecTimeOuts;
 			gConsecutiveTimeouts++)
 		{		
-			if (gAbortFlag > 0) {
+			if (gXferAbortFlag > 0) {
 abortRead:
 				SIGNAL(SIGINT, SIG_IGN);	/* Don't interrupt while aborting. */
 				SIGNAL(SIGPIPE, SIG_IGN);
@@ -369,12 +373,16 @@ abortRead:
 				 */
 				if ((xp->netMode == kNetReading) && (xp->outStream != gStdout))
 					SetLocalFileTimes(xp->doUTime, xp->remoteModTime, xp->localFileName);
-				if (gAbortFlag == SIGPIPE) {
+				if (gXferAbortFlag == SIGPIPE) {
 					if (gDebug)
 						EPrintF("\r** Broken pipe **\n");
-				} else
+				} else if (gXferAbortFlag == SIGINT) {
 					EPrintF("\r** Aborting Transfer **\n");
-				DebugMsg("Read timeouts: %d;  Write timeouts: %d;  Block timeout len: %d.\n", gNumReadTimeouts, gNumWriteTimeouts, gBlockTimeoutLen);
+				} else {
+					EPrintF("\r** Aborting Transfer (%d) **\n", gXferAbortFlag);
+				}
+				DebugMsg("Read timeouts: %d;  Write timeouts: %d.\n",
+					gNumReadTimeouts, gNumWriteTimeouts);
 				AbortDataTransfer(xp);
 				goto doneXfer;	/* Not reached. */
 			}
@@ -401,9 +409,11 @@ abortRead:
 			
 			if (nRead == -1L) {
 				/* A generic error. */
-				if (errno != EINTR)
+				if (errno != EINTR) {
 					Error(kDoPerror, "Error occurred during read!\n");
-				goto abortRead;
+					goto abortRead;
+					/* Note: You often get EINTR when you use ^Z. */
+				}
 			}
 			
 			/* else nRead == kTimeoutErr */
@@ -439,7 +449,7 @@ readOkay:
 			gConsecutiveTimeouts++)
 		{
 			/* This is the same glob of code as for the abortRead part. */
-			if (gAbortFlag > 0) {
+			if (gXferAbortFlag > 0) {
 abortWrite:
 				SIGNAL(SIGINT, SIG_IGN);	/* Don't interrupt while aborting. */
 				SIGNAL(SIGPIPE, SIG_IGN);
@@ -450,12 +460,16 @@ abortWrite:
 				 */
 				if ((xp->netMode == kNetReading) && (xp->outStream != gStdout))
 					SetLocalFileTimes(xp->doUTime, xp->remoteModTime, xp->localFileName);
-				if (gAbortFlag == SIGPIPE) {
+				if (gXferAbortFlag == SIGPIPE) {
 					if (gDebug)
 						EPrintF("\r** Broken pipe **\n");
-				} else
+				} else if (gXferAbortFlag == SIGINT) {
 					EPrintF("\r** Aborting Transfer **\n");
-				DebugMsg("Read timeouts: %d;  Write timeouts: %d;  Block timeout len: %d.\n", gNumReadTimeouts, gNumWriteTimeouts, gBlockTimeoutLen);
+				} else {
+					EPrintF("\r** Aborting Transfer (%d) **\n", gXferAbortFlag);
+				}
+				DebugMsg("Read timeouts: %d;  Write timeouts: %d.\n",
+					gNumReadTimeouts, gNumWriteTimeouts);
 				AbortDataTransfer(xp);
 				goto doneXfer;	/* Not reached. */
 			}
@@ -480,9 +494,12 @@ abortWrite:
 				if (xp->doReports)
 					ProgressReport(xp, kForceUpdate);
 			} else /* if (nPut == -1L or 0L) */ {
-				if ((errno != EINTR) && (errno != EPIPE))
+				if (errno == EPIPE) {
+					goto abortWrite;
+				} else if (errno != EINTR) {
 					Error(kDoPerror, "Error occurred during write!\n");
-				goto abortWrite;
+					goto abortWrite;
+				} /* else EINTR, fall-through and retry write. */
 			}
 			/* else nPut == kTimeoutErr, which we just continue. */
 		}	/* End write-block loop */
